import os
import sys
import hashlib
import string
import shutil
import time
class libyami:
    decodepsnr = 40
    encodepsnr = 33
    decodefiltes = ['h264','264','26l','jsv','jvt','avc','ivf','jpeg','mjpg','jpg']
    encodefiltes = ['I420','NV12','yuv']
    def __init__(self, decode, encode, psnr):
        self.decode = decode
        self.encode = encode
        self.psnr   = psnr
        self.failnumber  = 0
        self.opennumber  = 0
        self.passnumber  = 0
        self.passlist = []
        self.faillist = []
    def setdeencode(self, decode, encode):
        self.decode = decode
        self.encode = encode
    def setcompmode(self, compmode):
        self.compmode = compmode
    def setplaymode(self, playmode):
        self.playmode = playmode
    def setsavemode(self, savemode):
        self.savemode = savemode
    def setcodemode(self, codemode):
        self.codemode = codemode
    def setmd5mode(self, refmd5, mode):
        self.refmd5 = refmd5
        self.hasmd5 = mode
    def setrefmd5(self, refmd5):
        self.refmd5 = refmd5
    def setcurrentinput(self, currentinput):
        self.currentinput = currentinput
        self.currentinputbase = os.path.basename(currentinput)
    def setcurrentoutput(self, currentoutput):
        self.currentoutput = currentoutput
    def setoutputdir(self, outputdir):
        if not os.path.exists(outputdir) or not os.path.isdir(outputdir):
            os.mkdir(outputdir)
        self.outputdir = os.path.abspath(outputdir)
    def yamiplay(self, mediafile, outputdir, codemode, playmode, savemode, compmode):
        self.setcodemode(codemode)
        self.setsavemode(savemode)
        self.setplaymode(playmode)
        self.setcompmode(compmode)
        if os.path.isfile(mediafile):
            self.play(mediafile, outputdir)
        for root,dirs,files in os.walk(mediafile):
            for testfile in files:
                testfile = os.path.join(root, testfile)
                if os.path.isfile(testfile):
                    self.play(testfile, outputdir)
        if codemode == 'E' and self.compmode:
            self.parsepsnrresult()
    def play(self, mediafile, outputdir):
        self.setcurrentinput(mediafile)
        self.setoutputdir(outputdir)
        if self.codemode == 'D' and self.passfilter(mediafile, libyami.decodefiltes):
            self.opennumber += 1
            self.decodetest()
        elif self.codemode == 'E' and self.passfilter(mediafile, libyami.encodefiltes):
            self.opennumber += 1
            self.encodetest()
    def decodetest(self):
        message = ['']
        if not self.savemode:
            self.setcurrentoutput(os.path.join(self.outputdir, self.currentinputbase))
        else:
            self.setcurrentoutput(self.outputdir)
        for i in range(1, len(self.playmode)):
            if self.decodefunc(self.playmode[0], self.playmode[i], message):
                if self.playmode[0] == 'm' and self.playmode[i]== '0' and self.compmode:
                    self.writetologlist(message[0], self.comparemd5(0))
                elif self.playmode[0] == 'm' and self.playmode[i]== '-2' and self.compmode:
                    self.writetologlist(message[0], self.comparemd5(-2))
                else:
                    self.writetologlist(message[0], True)
            else:
                self.writetologlist(message[0], False)
            if not self.savemode and os.path.exists(self.currentoutput):
                os.remove(self.currentoutput)
    def encodetest(self):
        message = ['']
        self.setcurrentoutput(os.path.join(self.outputdir, self.currentinputbase+'.h264'))
        if not self.encodefunc(message):
            self.writetologlist(message[0], False)
            return
        if not self.compmode:
            os.remove(self.currentoutput)
            self.writetologlist(message[0], True)
            return
        rawI420 = self.currentinput
        self.setcurrentinput(self.currentoutput)
        self.setcurrentoutput(os.path.join(self.outputdir, os.path.basename(rawI420)))
        if not self.decodefunc('m', '0', message):
            return
        self.setcurrentinput(rawI420)
        self.psnrfunc()
        if not self.savemode:
            os.remove(self.currentoutput)
            os.remove(self.currentoutput+'.txt')
            os.remove(self.currentoutput+'.h264')
    def decodefunc(self, mode, modevalue, message):
        if mode == 'm' and modevalue == '0':
            decodeCmd = self.decode+' -i '+self.currentinput+' -'+mode+' '+modevalue+' -o '+self.currentoutput
        else:
            decodeCmd = self.decode+' -i '+self.currentinput+' -'+mode+' '+modevalue + ' -w 0'
        message[0] = self.currentinputbase + '\r'
        if os.system(decodeCmd) != 0:
            return False
        return True
    def encodefunc(self, message):
        W_H = self.get_w_h(self.currentinputbase)
        encodeCmd = self.encode+' -i '+self.currentinput+' -s I420 -W '+str(W_H[0])+' -H '+str(W_H[1])+' -o '+self.currentoutput
        message[0] = os.path.basename(self.encode) + '\r'
        if os.system(encodeCmd) != 0:
            return False
        return True
    def psnrfunc(self):
        W_H = self.get_w_h(self.currentinputbase)
        psnrCmd = self.psnr+' -i '+self.currentinput+' -o '+self.currentoutput+' -W '+str(W_H[0])+' -H '+str(W_H[1])+' -s '+str(libyami.encodepsnr)
        os.system(psnrCmd)
    def get_w_h(self, filename):
        filenamelist = filename.split('_')
        filenamelist = filenamelist[len(filenamelist) - 1].split('.')
        W_H = filenamelist[0].split('x')
        return W_H
    def comparemd5(self, mode):
        currentmd5 = self.getmd5sum(mode)
        return self.verifymd5(currentmd5)
    def getmd5sum(self, mode):
        if mode == 0: #calc md5 with dumped file
            fmd5value = hashlib.md5()
            fd = open(self.currentoutput, 'rb')
            fmd5value.update(fd.read())
            fd.close()
            return fmd5value.hexdigest()
        else:
            md5file = os.path.basename(self.currentinput) + '.md5'
            fd = open(md5file, 'r')
            for line in fd.readlines():
                line = line.strip('\n')
                if 'whole' in line:
                    strlist = line.split()
                    break
            fd.close()
            return strlist[4]
    def verifymd5(self, md5value):
        returnvalue = False
        refmd5 = os.path.join(os.path.dirname(self.currentinput),'bits.md5')
        fd = open(refmd5, 'r')
        lines = fd.readlines()
        for strline in lines:
            if md5value in strline and self.currentinputbase in strline:
                returnvalue = True
                break
        fd.close()
        return returnvalue
    def passfilter(self, mediafile, filters):
        for i in range(0, len(filters)):
            if mediafile.endswith(filters[i]):
                return True
        return False
    def writetologlist(self, message, checkvalue):
        if checkvalue:
            self.passlist.append(message)
            self.passnumber += 1
        else:
            self.faillist.append(message)
            self.failnumber += 1
        md5file = os.path.basename(self.currentinput) + '.md5'
        if os.path.exists(md5file):
            os.remove(md5file)
    def parsepsnrresult(self):
        psnrfile = os.path.join(self.outputdir, 'jpg_psnr.txt')
        if not os.path.exists(psnrfile):
            return
        fpsnr = open(psnrfile, 'r')
        lines = fpsnr.readlines()
        for line in lines:
            line = line[0:line.find(':')]
            if 'fail' in line:
                self.writetologlist(line, False)
            else:
                self.writetologlist(line, True)
        fpsnr.close()
    def writetologfile(self):
        logdirectory = os.path.join(os.getcwd(), 'log')
        if not os.path.exists(logdirectory):
            os.makedirs(logdirectory)
        logfile = 'test-result-'+time.strftime('%Y-%m-%d-%H-%M-%S')+'.log'
        logfile = os.path.join(logdirectory, logfile)
        logfilehandler = open(logfile, 'a')
        logfilehandler.write('open file:'+str(self.opennumber)+'\npass test:'+str(self.passnumber)+'\nfail test:'+str(self.failnumber))
        logfilehandler.write("\n\nfail test as follow:\n")
        for i in range(len(self.faillist)):
            logfilehandler.write(self.faillist[i]+'\n')
        logfilehandler.write("\n\npass test as follow:\n")
        for i in range(len(self.passlist)):
            logfilehandler.write(self.passlist[i]+'\n')
        logfilehandler.close()



#!/usr/bin/env python3

#This script will compare ssim value between libyami and ffmpeg.
#Firstly, it will generate yuv using yamidecode, then it will compare the ssim with ffmpeg. Mini version for ffmpeg is 2.8
#Example commands:
#    ssim_test.py directory
#    ssim_test.py file

import subprocess
import re
import glob, os
import sys
from os.path import join, getsize

def isCandidate(f):
    filename, ext = os.path.splitext(f)
    ext = ext.lower()
    supported = [
        ".jpeg"
        , ".jpg"
        , ".vc1"
        , ".rcv"
        , ".mpeg"
        , ".mpg"
        , ".mpeg2"
        , ".mjpg"
        , ".mjpeg"
    ]
    return ext in supported

def testYami(f, log):
    output = None if log else subprocess.DEVNULL
    yamidecode = os.path.dirname(os.path.realpath(__file__)) + "/../tests/yamidecode"
    r = subprocess.call(yamidecode + r" -i "+ f + " -f i420 -m 0 ", stdout=output,  stderr=output, shell=True)
    if r != 0:
        print("yami return "+str(r));
    return r == 0

def getFmt(f):
    filename, ext = os.path.splitext(f)
    if ".I420" == ext:
        return "yuvj420p"
    if ".422H" == ext:
        return "yuvj422p"
    if ".422V" == ext:
        return "yuvj440p"
    if ".444P" == ext:
        return "yuvj444p"
    return ""

def getYuvFile(src):
    dirname, basename = os.path.split(src)
    files = glob.glob(basename+"_*")
    if len(files) == 0:
        print("no yuv for "+src)
        return None
    return files[0]

def getFileInfo(file):
    m=re.search("(?P<width>\d+)x(?P<height>\d+)", file);
    w = int(m.group("width"))
    h = int(m.group("height"))
    if w == 0 or h == 0:
        return 0, 0, ""
    return w, h, getFmt(file)

def info(cmd, log):
    a = " All:"
    for line in cmd.stdout:
        l = line.decode("utf8");
        if (log):
            print(l)
        if "error" in l.lower():
            print(l)
            return False
        if  a in l:
            t =  l.split(a)
            t = t[1].split(" ")
            if (float(t[0]) > 0.99):
                return True;
    return False;

def verify(yuv, f, log):

    w, h, fmt = getFileInfo(yuv)
    cmd = subprocess.Popen(r"ffmpeg -f rawvideo -video_size " + str(w) + "x" + str(h) +" -pix_fmt "+ fmt + " -i " + yuv + " -i " + f + ' -lavfi "ssim;[0:v][1:v]psnr" -f null -', shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    return info(cmd, log)

def test(f, log = False):
    if not testYami(f, log):
        return False

    yuv = getYuvFile(f);
    if yuv is None:
        return False
    ret = verify(yuv, f, log)
    os.remove(yuv)
    print(f, end =  " ")
    print("pass" if ret else "failed")
    return ret

if len(sys.argv)!=2:
    print(sys.argv[0] + " directory")

dir = sys.argv[1]
#test file
if os.path.isfile(dir):
    test(dir, True)
    sys.exit(0)

#test directory
total = 0
failed = 0
for root, dirs, files in os.walk(dir):
    for f in files:
        if isCandidate(f):
            total += 1
            pss = test(join(root, f))
            failed +=  not pss
print("total = "+ str(total) +", failed = "+ str(failed))



#! /usr/bin/env python
import os
import sys
import shutil
from yamilib import libyami
if __name__ == '__main__':
    if len(sys.argv) < 5:
        print('Pls autotest.py file|dirctory decode|encode|v4l2 outdir savemode')
        sys.exit(-1)
    if not os.path.exists(sys.argv[1]):
        print(sys.argv[1] + ' does not exist')
        sys.exit(-1)
    mediafiles = os.path.abspath(sys.argv[1])
    testformat = sys.argv[2]
    testoutput = sys.argv[3]
    savemode = True if int(sys.argv[4])==1 else False
    autotestpath = os.path.dirname(os.path.abspath(sys.argv[0]))
    yamitestpath = os.path.join(autotestpath[0:autotestpath.rfind('/')],'tests')
    yamidecode = os.path.join(yamitestpath, 'yamidecode')
    yamiencode = os.path.join(yamitestpath, 'yamiencode')
    v4l2decode = os.path.join(yamitestpath, 'v4l2decode')
    v4l2encode = os.path.join(yamitestpath, 'v4l2encode')
    capidecode = os.path.join(yamitestpath, 'decodecapi')
    capiencode = os.path.join(yamitestpath, 'encodecapi')
    psnr       = os.path.join(autotestpath, 'psnr')
    decodemodelist = ['m', '0', '-2']
    encodemodelist = ['m', '0']
    v4l2wmodelist  = ['w', '0']
    v4l2mmodelist  = ['m', '0', '3', '4']

    if testformat == 'decode':
        yamiobject = libyami(yamidecode, yamiencode, psnr)
        yamiobject.yamiplay(mediafiles, testoutput, 'D', decodemodelist, savemode, not savemode)
        yamiobject.writetologfile()
    elif testformat == 'encode':
        yamiobject = libyami(yamidecode, yamiencode, psnr)
        yamiobject.yamiplay(mediafiles, testoutput, 'E', encodemodelist, savemode, True)
        yamiobject.writetologfile()
    elif testformat == 'v4l2':
        yamiobject = libyami(yamidecode, yamiencode, psnr)
        yamiobject.yamiplay(mediafiles, testoutput, 'D', v4l2wmodelist, savemode, False)
        yamiobject.yamiplay(mediafiles, testoutput, 'D', v4l2mmodelist, savemode, False)
        yamiobject.yamiplay(mediafiles, testoutput, 'E', v4l2mmodelist, savemode, False)
        yamiobject.setdeencode(capidecode, capiencode)
        yamiobject.yamiplay(mediafiles, testoutput, 'D', v4l2wmodelist, savemode, False)
        yamiobject.yamiplay(mediafiles, testoutput, 'D', v4l2mmodelist, savemode, False)
        yamiobject.yamiplay(mediafiles, testoutput, 'E', v4l2mmodelist, savemode, False)
        yamiobject.setdeencode(v4l2decode, v4l2encode)
        yamiobject.yamiplay(mediafiles, testoutput, 'D', v4l2wmodelist, savemode, False)
        yamiobject.yamiplay(mediafiles, testoutput, 'D', v4l2mmodelist, savemode, False)
        yamiobject.yamiplay(mediafiles, testoutput, 'E', v4l2mmodelist, savemode, False)
        yamiobject.writetologfile()
